#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/cursorfont.h>

#include "pixmap_io.h"

/*****************************************************************************/

#define BAR 20
#define MAX_BUTTON 5
Display *dpy;
int scr;
Window top, win;
GC gc;
int red, yellow;
XEvent evt;
int b_count;
XFontStruct *font;
struct {
  char *label;
  Window win;
} buttons[ MAX_BUTTON ];
int b_right;

/*****************************************************************************/

#define NB_GRAY	64
unsigned char *data_in, *pixels;
int width, height;
XImage *image;
int LUT[NB_GRAY+2];
int count = 0;

void init_data()
/* convertit les niveaux de gris en numero de pixel */
{
  long size;
  unsigned char *ptr_in, *ptr_pix;

  size = (long)width*height;
  ptr_in = data_in;
  ptr_pix = pixels;
  for( ; size > 0; size--, ptr_in++, ptr_pix++ )
    *ptr_pix = LUT[ ((int)*ptr_in + 256/NB_GRAY - 1) * NB_GRAY / 254 ];
}

void draw_win()
{
  char s[20];

  XPutImage( dpy, win, gc, image, 0, 0, 0, 0, width, height);
  sprintf( s, "%d", count++ );
  XDrawString( dpy, win, gc, 10, 10, s, strlen(s) );
}

/*****************************************************************************/

void add_button( char *label )
{
  Window win;
  int size;

  size =  XTextWidth( font, label, strlen( label ) ) + BAR;
  win = XCreateSimpleWindow( dpy, top,
			    b_right, 0, size, BAR - 1, 1,
			    BlackPixel( dpy, scr ), WhitePixel( dpy, scr ) );
  XSelectInput( dpy, win, ExposureMask + ButtonPressMask );
  XMapWindow( dpy, win );
  buttons[ b_count ]. win = win;
  buttons[ b_count ]. label = label;
  b_count++;
  b_right = b_right + size;
}

void draw_button( int id )
{
  if( (id >= 0) && (id < b_count) )
  XDrawString( dpy, buttons[ id ]. win, DefaultGC( dpy, scr ),
	      BAR / 2, ( BAR + font->ascent - font->descent ) / 2,
	      buttons[ id ]. label, strlen( buttons[ id ]. label ) );
}

int find_button( Window win )
{
  int ret;

  for( ret = 0; ret < b_count; ret++ )
    if( buttons[ ret ]. win == win )
      return( ret );
  return( -1 );
}

/*****************************************************************************/

void initX( char *display, int dim_x, int dim_y, char *wName )
{
  XColor col, colb;
  Colormap map;
  int i;

  dpy = XOpenDisplay( display );
  if( dpy == (Display *) 0 )
    {
      printf( "can't open display : %s\n", display );
      exit(0);
    }
  scr = DefaultScreen( dpy );
  top = XCreateSimpleWindow( dpy, RootWindow( dpy, scr ),
			    0, 0, dim_x, dim_y + BAR, 1,
			    BlackPixel( dpy, scr ), WhitePixel( dpy, scr ) );
  XSelectInput( dpy, top, StructureNotifyMask );
  XStoreName( dpy, top, wName );

  pixels = malloc( (long)width*height );
  image = XCreateImage( dpy, DefaultVisual( dpy, scr ), 
                       DefaultDepth( dpy, scr ), ZPixmap, 0, 
                       pixels, dim_x, dim_y, 8, dim_x);

  map = DefaultColormap( dpy, scr );
  if( XAllocNamedColor( dpy, map, "red", &col, &colb ) )
    red = col. pixel;
  else
    red = WhitePixel( dpy, scr );
  if( XAllocNamedColor( dpy, map, "yellow", &col, &colb ) )
    yellow = col. pixel;
  else
    yellow = WhitePixel( dpy, scr );
  LUT[0] = BlackPixel( dpy, scr );
  LUT[NB_GRAY+1] = WhitePixel( dpy, scr );
  for( i = 1; i <= NB_GRAY; i++)
    {
    col. red = col. green = col. blue = 65535UL * i / ( NB_GRAY + 1 );
    if( XAllocColor( dpy, map, &col ) )
      LUT[i] = col. pixel;
    else
      LUT[i] = red;
    }

  gc = XCreateGC( dpy, top, 0, (XGCValues *)0 );
  font = XLoadQueryFont( dpy, "fixed" );
  XSetFont( dpy, gc, font->fid );
/*  XSetForeground( dpy, gc, WhitePixel( dpy, scr )); */
  XSetForeground( dpy, gc, yellow );
  b_count = 0;
  b_right = 0;

  win = XCreateSimpleWindow( dpy, top,
			    0, BAR, dim_x, dim_y, 1,
			    WhitePixel( dpy, scr ), BlackPixel( dpy, scr ) );
  XSelectInput( dpy, win, ExposureMask );
  XDefineCursor( dpy, win, XCreateFontCursor( dpy, XC_crosshair ) );

  XMapWindow( dpy, top );
  XMapWindow( dpy, win );

  add_button( "quit" );
  add_button( "glop" );
  XFlush( dpy );
}

void resize( width, height )
int width, height;
{
  XWindowChanges values;

  values. width = width;
  values. height = height;
  XConfigureWindow( dpy, win, CWWidth + CWHeight, &values );
}

void main_loop()
{
  while( 1 )
    {
      XNextEvent( dpy, &evt );
      switch( evt. type )
	{
	case Expose:
	  if( evt.xexpose.window == win )
	    draw_win();
	  else
	    draw_button( find_button( evt.xexpose.window ) );
	  break;
	case ButtonPress:
	  switch( find_button( evt.xbutton.window ) )
	    {
	    case 0:
	      exit(0);
	    case 1:
	      XClearWindow( dpy, win );
/*	      read_file( file );*/
	      if(0)
                XUnmapWindow( dpy, evt.xbutton.window );
	      else
		draw_win();
	    }
	  break;
	case ConfigureNotify:
	  resize( evt.xconfigure.width, evt.xconfigure.height );
	  break;
	case MapNotify:
	case ReparentNotify:
	  break;
	default:
	  printf("evt %d\n", evt. type);
	}
    }
}

/*****************************************************************************/

int main( int argc, char **argv )
{
  char *display = NULL;

  display = NULL;
  if( (argc > 1) && ((data_in = load_pixmap(argv[1], &width, &height)) != NULL) )
    {
    initX( display, width, height, "GLOP" );
    init_data();
    main_loop();
    }
  return 0;
}



